from generator.yaml import get_yaml, merge_yaml_strings


def test_get_yaml():
    name: str = 'test'
    variables: dict = {
        'mandatories': [{'name': 'mandatories', 'description': 'mandatories'}],
        'optionals': [
            {
                'name': 'optionals',
                'description': 'optionals',
                'default': 'optional_value',
            }
        ],
        'nullables': [{'name': 'nullables', 'description': 'nullables'}],
    }
    resutls = get_yaml(name, variables)
    print(resutls)
    excepted = """test:
  enabled: true
  # mandatories - mandatories
  mandatories: 
  # optionals - optionals
  # optionals: "optional_value"
  # nullables - nullables
  # nullables: 
"""
    assert resutls == excepted


def test_merge_yaml_strings():
    yaml1 = """gke:
  cluster:
    enabled: true
    # required - required value
    required:
    # optional - optional value
    # optional: "optional"
    # nullable - nullable value
    # nullable:
"""

    yaml2 = """gke:
  helper:
    enabled: true
    # required - required value
    required:
    # optional - optional value
    # optional: "optional"
    # nullable - nullable value
    # nullable:
"""

    result = merge_yaml_strings(yaml1, yaml2)

    assert "cluster:" in result
    assert "helper:" in result
    assert "optional: \"optional\"" in result


def test_merge_yaml_strings_with_blank_lines_inside_blocks():
    yaml1 = """gke:
  cluster:
    enabled: true

    # required - required value
    required:

    # optional - optional value
    # optional: "optional"
"""
    yaml2 = """gke:
  cluster:
    # additional setting
    # another: "value"

  helper:
    enabled: true
"""

    result = merge_yaml_strings(yaml1, yaml2)

    assert "enabled: true\n\n    # required - required value" in result
    assert "helper:" in result
    assert "another: \"value\"" in result


def test_merge_yaml_strings_with_other_keys():
    yaml1 = """# Generated by terragrunt-generator
# Do not edit manually

test:
  cluster:
    enabled: true
"""
    yaml2 = """gke:
  helper:
    enabled: true
"""

    result = merge_yaml_strings(yaml1, yaml2)

    assert "# Generated by terragrunt-generator" in result
    assert "test:" in result
    assert "cluster:" in result
    assert "gke:" in result
    assert "helper:" in result


def test_merge_yaml_strings_ends_with_block_without_trailing_line():
    yaml1 = """alpha:
  setting: true
beta:
  active: false"""
    yaml2 = """gamma:
  value: 123"""

    result = merge_yaml_strings(yaml1, yaml2)

    assert "alpha:" in result
    assert "beta:" in result
    assert "gamma:" in result
    assert "value: 123" in result
